########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../gl.4gm/i_chart.4gs/fg_funcs.4gl  1.1  Delta: 2/4/91
########################################################################

globals "globals.4gl"

######################################################################
function dup_key(p_acct_no)
# returns true/false
######################################################################
# this function checks for a duplicate key in another document 
# and will return a true is a duplicate is found.
#
    define
        p_rowid   integer,               # rowid of current document
        p_count   smallint,              # number of records found
        p_acct_no like stxchrtr.acct_no

    if dup_prep is null
    then
        let scratch = "select count(*) from stxchrtr ",
                      "where rowid != ? and acct_no = ?"
        prepare dup_stmt from scratch
        declare dup_curs cursor for dup_stmt
        let dup_prep = "Y"
    end if

    let p_rowid = get_scrlib("curs_rowid")
    open dup_curs using p_rowid, p_acct_no
    fetch dup_curs into p_count

    if p_count = 0 then return true end if

    call fg_err(1)  # Duplicate account number
    return false

end function 
# dup_key

#######################################################################
function setype()
#######################################################################
# - this function takes the account number and figures the acct type.

    if p_xchrtr.acct_no < m_xcntrc.fixed_asset then
        call setfld("CURRENT ASSETS","A","1","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.curr_liab then
        call setfld("FIXED ASSETS","A","2","N") return end if
    if p_xchrtr.acct_no < m_xcntrc.long_term_liab then
        call setfld("CUR LIABILITIES","B","3","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.capital then
        call setfld("L/T LIABILITIES","B","4","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.income then
        call setfld("CAPITAL","C","5","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.cost_goods then
        call setfld("INCOME","D","6","Y") return end if
    if p_xchrtr.acct_no < m_xcntrc.expense then
        call setfld("COST OF GOODS","E","7","N") return end if
    call setfld("EXPENSES","F","8","N")
end function

# setype()

#######################################################################
function setfld(acct_type,acct_cat,processing_seq,incr_with_crdt)
#######################################################################
#
    define
        acct_type   like stxchrtr.acct_type,
        acct_cat    like stxchrtr.acct_cat,
        processing_seq  like stxchrtr.processing_seq,
        incr_with_crdt  like stxchrtr.incr_with_crdt

    let p_xchrtr.acct_type = acct_type
#   display by name p_xchrtr.acct_type
    let m_xchrtr.acct_cat = acct_cat
    let m_xchrtr.processing_seq = processing_seq
    let p_xchrtr.incr_with_crdt = incr_with_crdt
#   display by name p_xchrtr.incr_with_crdt

end function
# setfld()

#######################################################################
function activity()
# returning true/false based on activity/no_activity posted.
#######################################################################
#
    define
        counter smallint

    # if account number is null, return false
    if p_xchrtr.acct_no is null then return false end if

    # check for activity in activity table
    select count(*) into counter from stgactvd
      where acct_no = p_xchrtr.acct_no
    if counter > 0 then return true end if

    # if gl is setup, then check for activity in detail lines
    if m_gcntrc.gl_balanced = "Y" 
    then 
        # check for activity in history table
        select count(*) into counter from stxchrtd
          where acct_no = p_xchrtr.acct_no
        if counter > 0 then return true end if
    end if

    # can't find any activity
    return false

end function
# activity()

#######################################################################
function fill_line()
# returning true/false based on activity/no_activity posted.
#######################################################################
#

    if p_xchrtd[p_cur].department is null
      then let nxt_fld = "department" return end if
    if p_xchrtd[p_cur].period_month is null
      then let nxt_fld = "period_month" return end if
    if p_xchrtd[p_cur].period_year is null
      then let nxt_fld = "period_year" return end if

    if budget_only() 
    then 
        # if future period, clear activity and balance
        if  p_xchrtd[p_cur].period_year > m_gcntrc.curr_year or
           (p_xchrtd[p_cur].period_year = m_gcntrc.curr_year and 
            p_xchrtd[p_cur].period_month > m_gcntrc.curr_period)
        then
            let p_xchrtd[p_cur].activity = 0
            let p_xchrtd[p_cur].balance = 0
            let p_xchrtd[p_cur].this_month = 0
            display p_xchrtd[p_cur].activity, p_xchrtd[p_cur].balance, 
                    p_xchrtd[p_cur].this_month
                 to s_chrtd[s_cur].activity, s_chrtd[s_cur].balance, 
                    s_chrtd[s_cur].this_month
        end if
        if scr_fld = "balance"
          then let nxt_fld = "period_year" return end if
        let nxt_fld = "budget"
    end if
 
end function
# fill_line

#######################################################################
function budget_only()
# returning true/false based on ability to enter budget amounts only
#######################################################################
#  This function returns true if the user is only allowed entry
#  into the budget field of the line.
#  If the period is not g/l controlled, and is in the future,
#  then allow entry into budget field only.
#
   define
       period like stxperdr.period,
       period_year like stxperdr.period_year

    let period = p_xchrtd[p_cur].period_month
    let period_year = p_xchrtd[p_cur].period_year

    # if period is unknown then the jury is still out.
    if period is null or period_year is null then return false end if

    if gl_controlled()
    then
        return true
    else
        if  period_year < m_gcntrc.curr_year
         or (period_year = m_gcntrc.curr_year
             and period <= m_gcntrc.curr_period)
          then return false
          else return true
        end if
    end if

end function
# budget_only()

#######################################################################
function gl_controlled()
#   returns true if this period is being controlled by g/l
#######################################################################
#  This function checks the "balanced" field for this period in
#  the stxperdr file to see if it is under g/l control (balanced=Y).
#  Returns true if balanced = "Y"
#  If cannot find the period in the perdr talbe, it's not controlled
#  If a disk error occurs report the error, and the safest thing
#  to do is to assume the period is under g/l control.
#
   define
       period like stxperdr.period,
       period_year like stxperdr.period_year

    let period = p_xchrtd[p_cur].period_month
    let period_year = p_xchrtd[p_cur].period_year

    # not being controlled by g/l if unknown period
    if period is null or period_year is null then return false end if

    # Prepare the SQL statement if needed
    if l_prepared = false
    then
        whenever error continue
        prepare get_perd from
          "select * from stxperdr where period = ? and period_year = ?"
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            call fg_err(7) # Period definition table is corrupt
            return true    # assume period is g/l controlled
        end if
        declare perd_curs cursor for get_perd
        let l_prepared = true
    end if

    # read the m_xperdr record in from disk if not current.
    if   period = m_xperdr.period
     and period_year = m_xperdr.period_year
    then else
       whenever error continue
       open perd_curs using period, period_year
       fetch perd_curs into m_xperdr.*
       whenever error call error_handler
       # if unknown error, assume period is controlled by g/l
       if sqlca.sqlcode < 0 then return true end if
       # if not found, then period is not controlled by g/l
       if sqlca.sqlcode = notfound then return false end if
    end if

    if m_xperdr.balanced = "Y"
      then return true
      else return false
    end if

end function
# gl_controlled()

#######################################################################
function datechk()
# converts dates, checks for gl_controlled flag            
#######################################################################
#                                                              

    define
        n smallint

    # check for a 1 digit month entered, and convert it to 2 digits
    let n = p_xchrtd[p_cur].period_month
    if n < 1 then let n = 1 end if  # no 0 periods entered
    if n is not null
    then
        let p_xchrtd[p_cur].period_month = n using "&&"
        display p_xchrtd[p_cur].period_month 
          to s_chrtd[s_cur].period_month
    end if

    let n = p_xchrtd[p_cur].period_year
    # check for a 2 digit year, and convert to 4 digit
    if n is not null and n < 1000
    then
        if n > 50
        then 
            let p_xchrtd[p_cur].period_year = 
                (n + 1900) using "&&&&"
        else 
            let p_xchrtd[p_cur].period_year = 
                (n + 2000) using "&&&&"
        end if
        display p_xchrtd[p_cur].period_year 
          to s_chrtd[s_cur].period_year
    end if

    let n = false  # temporary variable for error condition
    if chrtdup()
    then
        call fg_err(5) # Duplicate department and period
        let n = true
    end if
    if gl_controlled()
    then
        call fg_err(8) # Cannot add history into a G/L controlled period
        let n = true
    end if
    if n = true  # error condition: re-enter period dates
    then
        let p_xchrtd[p_cur].period_month = null
        let p_xchrtd[p_cur].period_year = null
        display
            p_xchrtd[p_cur].period_month, p_xchrtd[p_cur].period_year
          to
            s_chrtd[s_cur].period_month, s_chrtd[s_cur].period_year
        let nxt_fld = "period_month"
    end if

end function
# datechk()
 
###################################################################
function chrtdup()
###################################################################
# returns a true if there is a duplicate entry of dept-period/year
#
    define
        n   smallint    # working number

    if  p_xchrtd[p_cur].department is null
     or p_xchrtd[p_cur].period_month is null
     or p_xchrtd[p_cur].period_year is null
    then
        return false
    end if

    for n = 1 to arr_count()    # for all active array elements
        if p_xchrtd[n].department is not null 
          and n != p_cur
        then
           if   p_xchrtd[n].department =
                p_xchrtd[p_cur].department 
            and p_xchrtd[n].period_month =
                p_xchrtd[p_cur].period_month 
            and p_xchrtd[n].period_year =
                p_xchrtd[p_cur].period_year 
           then
               return true
           end if
        end if
    end for
    return false

end function
# chrtdup()

#######################################################################
function ok_delete()
# returning true/false for  ok/not_ok  to delete
#######################################################################
#
    if activity()
    then
        call fg_err(3) # Posted Data on File
        return false
    end if
    return true
end function
# ok_delete()
