######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    z_flag    smallint,
    num_in    smallint,
    num_out   smallint,
    lookup_prep char(1), # Have the lookups been prepared?
    insert_prep char(1), # Has the insert statement been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=row, 2=field
    tab_pressed smallint,# (boolean) was the tab key pressed?
    in_insert smallint   # (boolean) true if we're in 'insert row'


######################################################################
function lld_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    define
        goto_top  smallint   # (boolean) Go to the top of the array?

    # Trap fatal errors
    whenever error call error_handler

    # Set some initial values
    let goto_top = true

    # This loop provides the ability to goto the top of the array.
    # Logic contained within the loop is not indented for readability.
    while goto_top

    let goto_top = false
    let in_insert = false
    let exit_level = 0
    let tab_pressed = false
    call set_count(rec1_cnt)
    input array p_xperdr without defaults from s_perdr.*

      # The following section is for array type inputs
      before row
        if in_insert
        then
            # F1/F2 Scenario - fix parallel array if pre-informix 4.0
            if not is_4_0() and rec1_cnt > 0
              then call lld_a_delete() end if
            let in_insert = false  # Reset in_insert flag
        end if
        let scr_fld = ""           # Reset the current field
        let p_cur = arr_curr()     # Set the current array line
        let s_cur = scr_line()     # Set the current screen line
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_row()
        goto next_field
      after row
        call lld_showline()
        call lld_a_row()
        goto next_field
      before insert
        let in_insert = true
        let rec1_cnt = arr_count() # Set the number of array elements
        call lld_b_insert()
        goto next_field
      after insert
        let in_insert = false
        call lld_a_insert()
        goto next_field
      before delete
        call lld_b_delete()
        goto next_field
      after delete
        call lld_a_delete()
        goto next_field

      # All entry fields must have before and after field processing
      before field period call lld_b_field("stxperdr.period")
        goto next_field
      before field period_year call lld_b_field("stxperdr.period_year")
        goto next_field
      before field start_date call lld_b_field("stxperdr.start_date")
        goto next_field
      before field end_date call lld_b_field("stxperdr.end_date")
        goto next_field

      after field period call lld_a_field()
        goto next_field
      after field period_year call lld_a_field()
        goto next_field
      after field start_date call lld_a_field()
        goto next_field
      after field end_date call lld_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 2 then call lld_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after row' logic if necessary
        if exit_level >= 1 then call lld_a_row() end if
        if nxt_fld is not null then goto next_field end if

        # Don't run 'after input' logic if looping to top
        if goto_top then exit input end if

        # Run the 'after input' logic before exiting input
        call lld_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("lld_input")  # Map the key to the event
        call lld_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "period" next field period
          when nxt_fld = "period_year" next field period_year
          when nxt_fld = "start_date" next field start_date
          when nxt_fld = "end_date" next field end_date
          when nxt_fld = "exit input" goto end_input
          when nxt_fld = "goto top"
            call mld_arr_count()
            let goto_top = true
            goto end_input
        end case

    end input

    # This is the end of the 'goto top' loop.
    end while

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# lld_input()

######################################################################
function lld_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field and row level exit
    let exit_level = 2

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "period"
        call fgdb_period()
      when scr_fld = "period_year"
        call fgdb_period_year()
      when scr_fld = "start_date"
        call fgdb_start_date()
      when scr_fld = "end_date"
        call fgdb_end_date()
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# lld_b_field()

######################################################################
function lld_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # Require row level exit
    let exit_level = 1

    # After field logic
    case
      when scr_fld = "period"
        if p_xperdr[p_cur].period = "00"
        then
            let p_xperdr[p_cur].period = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "period"
            return
        end if
        if p_xperdr[p_cur].period is not null
          then call datechk() end if

      when scr_fld = "end_date"
        call fgda_end_date()

    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "period_year"
            call datechk()
        end case
    end if

end function
# lld_a_field()

######################################################################
function lld_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#

    let num_in = 0
    let num_out = 0
    for n = 1 to rec1_cnt
      if (p_xperdr[n].period is null or
          p_xperdr[n].period_year is null or
          p_xperdr[n].start_date is null or
          p_xperdr[n].end_date is null)
      then
          let num_out = num_out + 1
          continue for
      end if
      let num_in = num_in + 1
      let p_xperdr[num_in].* = p_xperdr[n].*
      let q_xperdr[num_in].* = q_xperdr[n].*
    end for
    initialize p_xperdr[num_in + 1].* to null
    initialize q_xperdr[num_in + 1].* to null
    for n = (num_in + 2) to rec1_cnt
      let p_xperdr[n].* = p_xperdr[num_in + 1].*
      let q_xperdr[n].* = q_xperdr[num_in + 1].*
    end for
    let rec1_cnt = num_in
    call lld_display()

    if not tab_pressed
    then
        if valid_periods() = false
          then let nxt_fld = "goto top" return end if
        if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "N"
        then
            if close_setup() = false
            then
                let p_gcntrc.gl_balanced = "N"
                call ok_cancel("Y")
                let int_flag = -1
            end if
        end if
    end if
end function
# lld_a_input()

######################################################################
function lld_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# lld_event()

######################################################################
function lld_b_row()
######################################################################
# This function is called before you enter a new row.
#

    if p_xperdr[p_cur].balanced = "Y"  # disable delete key
    then
      options delete key f36
      return
    else
      options delete key f2
    end if
end function
# lld_b_row()

######################################################################
function lld_a_row()
######################################################################
# This function is called whenever you leave a row.
#
    # No more exit levels required
    let exit_level = 0

end function
# lld_a_row()

######################################################################
function lld_b_insert()
######################################################################
# This function is called before a new row is added to the array.
#
    define
        n smallint   # Generic counter

    # Expand (shift) the parallel array
    for n = (rec1_cnt - 1) to p_cur step -1
        let q_xperdr[n+1].* = q_xperdr[n].*
    end for

    # Blank out the current array element
    initialize q_xperdr[p_cur].* to null

    let scr_fld = null
end function
# lld_b_insert()

######################################################################
function lld_a_insert()
######################################################################
# This function is called after a new row has been added to the array.
#

end function
# lld_a_insert()

######################################################################
function lld_b_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and before the actual array elements have been shifted.
#

end function
# lld_b_delete()

######################################################################
function lld_a_delete()
######################################################################
# This function is called when the user presses [F2] (delete a row),
# and after the actual array elements have been shifted.
# (rec1_cnt contains the number of elements before the delete)
#
    define
        n smallint   # Generic counter

    # Compress (shift) the parallel array
    for n = (p_cur + 1) to rec1_cnt
        let q_xperdr[n-1].* = q_xperdr[n].*
    end for

    # Blank out the last (duplicate) data element
    initialize p_xperdr[rec1_cnt].* to null
    initialize q_xperdr[rec1_cnt].* to null

    # Perform array math
    call llh_math()
    call llh_display()

end function
# lld_a_delete()

######################################################################
function lld_setdata()
######################################################################
# This function is called to set the this_data variable
#
    case
      when scr_fld = "period"
        call set_data(p_xperdr[p_cur].period)
      when scr_fld = "period_year"
        call set_data(p_xperdr[p_cur].period_year)
      when scr_fld = "start_date"
        call set_data(p_xperdr[p_cur].start_date)
      when scr_fld = "end_date"
        call set_data(p_xperdr[p_cur].end_date)
    end case
end function
# lld_setdata()

######################################################################
function lld_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "period"
        display p_xperdr[p_cur].period
          to s_perdr[s_cur].period attribute(reverse)
      when scr_fld = "period_year"
        display p_xperdr[p_cur].period_year
          to s_perdr[s_cur].period_year attribute(reverse)
      when scr_fld = "start_date"
        display p_xperdr[p_cur].start_date
          to s_perdr[s_cur].start_date attribute(reverse)
      when scr_fld = "end_date"
        display p_xperdr[p_cur].end_date
          to s_perdr[s_cur].end_date attribute(reverse)
    end case
end function
# lld_high()

######################################################################
function lld_showline()
######################################################################
# This function displays the current p_* variables onto the screen line
#
    display p_xperdr[p_cur].* to s_perdr[s_cur].*
end function
# lld_showline()

######################################################################
function lld_display()
######################################################################
# This function displays the first screenful of data
#
    for p_cur = 1 to scr1_max
        display p_xperdr[p_cur].* to s_perdr[p_cur].*  attribute(red)
    end for
end function
# lld_display()

######################################################################
function lld_p_prep(n)
######################################################################
# This function creates the p_[]* record from the m_* record
#
    define
        n smallint      # Array element to process

    let p_xperdr[n].period = m_xperdr.period
    let p_xperdr[n].period_year = m_xperdr.period_year
    let p_xperdr[n].start_date = m_xperdr.start_date
    let p_xperdr[n].end_date = m_xperdr.end_date
    let p_xperdr[n].balanced = m_xperdr.balanced

end function
# lld_p_prep()

######################################################################
function lld_add()
######################################################################
# This function inserts data into the detail table.
#
    # Declare an insert cursor for maximum efficiency
    if insert_prep is null
    then
        let insert_prep = "Y"
        declare d_insert cursor for
          insert into stxperdr values(m_xperdr.*)
    end if

    # Insert all array elements
    open d_insert
    for p_cur = 1 to rec1_cnt
        call lld_m_prep(p_cur)
        put d_insert
    end for
    close d_insert

end function
# lld_add()

