########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../gl.4gm/i_contrl.4gs/fg_funcs.4gl  1.2  Delta: 2/13/91
########################################################################

globals "globals.4gl"

define
    select_prep char(1), # Has the select statement been prepared?
    tmp_tbl_created char(1)  # flag

#######################################################################
function fgha_curr_period()
# contains the after field logic for header field curr_period
#######################################################################
#

    if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "Y"
      then let nxt_fld = "dir_db_cr" return end if

    # convert 1 digit period to 2 digits
    let n = p_gcntrc.curr_period
    if n < 1 then let n = 1 end if  # no 0 periods entered
    if n is not null
    then
        let p_gcntrc.curr_period = n using "&&"
        display by name p_gcntrc.curr_period
    end if

end function
# fgha_curr_period()

#######################################################################
function fghb_curr_year()
# contains the before field logic for header field curr_year
#######################################################################
#

    if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "Y"
      then let nxt_fld = "dir_db_cr" return end if
    # check for a 2 digit year, and convert to 4 digit
    let n = p_gcntrc.curr_year
    if n is not null and n < 1000
    then
        if n > 50
          then let p_gcntrc.curr_year = (n + 1900) using "&&&&"
          else let p_gcntrc.curr_year = (n + 2000) using "&&&&"
        end if
        display by name p_gcntrc.curr_year
    end if

end function
# fghb_curr_year()

#######################################################################
function fgha_retain_earnings()
# contains the after field logic for header field retain_earnings
#######################################################################
#
    define
        db_cr    char(1)

    if p_gcntrc.gl_balanced = "Y"
    then 
        # only check for the correct retained earnings
        # account if setup has already been done.
        call al_chrtn(p_gcntrc.retain_earnings)
            returning scratch, db_cr
        if scratch = "NOT FOUND"
        then
            call fg_err(8)    # Retained earnings account not found
            let nxt_fld = "retain_earnings"
        end if
    end if

end function
# fgha_retain_earnings()

#######################################################################
function fgha_gl_balanced()
# contains the after field logic for header field gl_balanced
#######################################################################
#

    if p_gcntrc.gl_balanced != "Y" and
       p_gcntrc.gl_balanced != "N"
    then
        call fg_err(3)     # G/L balanced must be a "Y" or "N""
        let p_gcntrc.gl_balanced = ""
        display by name p_gcntrc.gl_balanced
        let nxt_fld = "gl_balanced"
        return
    end if
    if p_gcntrc.gl_balanced = "Y"
      then let nxt_fld = "retain_earnings" end if

end function
# fgha_gl_balanced()

#######################################################################
function fgdb_period()
# contains the after field logic for header field period
#######################################################################
#

    # set the default
    if p_xperdr[p_cur].period is null
    then
        if p_cur > 1
        then
            let n = p_xperdr[p_cur - 1].period + 1
            if n < 13
            then
                let p_xperdr[p_cur].period = n using "&&"
            end if
        end if
    end if

end function
# fgdb_period()

#######################################################################
function fgdb_period_year()
# contains the after field logic for header field period_year
#######################################################################
#

    if p_xperdr[p_cur].balanced = "Y" 
      then let nxt_fld = "period" return end if

    # set the default
    if p_xperdr[p_cur].period_year is null and p_cur > 1
    then
        case
          when p_xperdr[p_cur].period = p_xperdr[p_cur - 1].period + 1
            let p_xperdr[p_cur].period_year =
                p_xperdr[p_cur - 1].period_year
          when p_xperdr[p_cur].period = "01"
            let p_xperdr[p_cur].period_year =
                (p_xperdr[p_cur - 1].period_year + 1) using "&&&&"
        end case
    end if

end function
# fgdb_period_year()

#######################################################################
function fgdb_start_date()
# contains the after field logic for header field start_date
#######################################################################
#
    
    if p_xperdr[p_cur].period is null
      then let nxt_fld = " period" return end if
    if p_xperdr[p_cur].period_year is null
      then let nxt_fld = "period_year" return end if
    if p_xperdr[p_cur].balanced = "Y" 
      then let nxt_fld = "period" return end if

    # set the default to the previous period's end_date + 1
    if p_xperdr[p_cur].start_date is null
    then
        let per = p_xperdr[p_cur].period
        let yr  = p_xperdr[p_cur].period_year
        if per = "01"
        then
            let yr = yr - 1
            let per = "**"     # asterisks sort ahead of numbers
            for n = 1 to arr_count()
                if   p_xperdr[n].period_year = yr
                 and p_xperdr[n].period > per
                then
                    let per = p_xperdr[n].period
                end if
            end for
        else
            let per = (per - 1) using "&&"
        end if

        # per and yr are set to last period
        initialize m_xperdr.* to null
        for n = 1 to arr_count()
            if   p_xperdr[n].period_year = yr
             and p_xperdr[n].period = per
            then
                let m_xperdr.* = p_xperdr[n].*
            end if
        end for
        if m_xperdr.end_date is not null
        then
            let p_xperdr[p_cur].start_date = m_xperdr.end_date + 1
            display p_xperdr[p_cur].start_date 
              to s_perdr[s_cur].start_date
            let nxt_fld = "end_date"
        end if
    end if      # null start_date
 
end function
# fgdb_start_date()

#######################################################################
function fgdb_end_date()
# contains the after field logic for header field end_date
#######################################################################
#

    if p_xperdr[p_cur].period is null
      then let nxt_fld = "period" return end if
    if p_xperdr[p_cur].period_year is null
      then let nxt_fld = "period_year" return end if
    if p_xperdr[p_cur].balanced = "Y" 
      then let nxt_fld = "period" return end if

    if   p_xperdr[p_cur].end_date is null
     and p_xperdr[p_cur].start_date is not null
    then
        let scratch = p_xperdr[p_cur].start_date using "mmddyyyy"
        let yr = scratch[5,8]
        let per = scratch[1,2]
        if per = "12"
        then
            let per = "01"
            let yr = (yr + 1) using "&&&&"
        else
            let per = (per + 1) using "&&"
        end if
        let scratch = per clipped, "/01/", yr[3,4]

        # m_xperdr.end_date contains the proposed
        # next period start date
        let m_xperdr.end_date = scratch
        let p_xperdr[p_cur].end_date = m_xperdr.end_date - 1
    end if

end function
# fgdb_end_date()

#######################################################################
function fgda_end_date()
# contains the after field logic for header field end_date
#######################################################################
#

    if   p_xperdr[p_cur].end_date is not null
     and p_xperdr[p_cur].start_date is not null
     and p_xperdr[p_cur].end_date < p_xperdr[p_cur].start_date
    then
        call fg_err(7)     # End date cannot be before start date.
        let nxt_fld = "end_date"
    end if

end function
# fgda_end_date()

#######################################################################
function datechk()
# logic invoked after entry to period and perod_year
#######################################################################
#

    if p_xperdr[p_cur].balanced = "Y"
    then
        let p_xperdr[p_cur].period = prev_data[1,2]
        display p_xperdr[p_cur].period to s_perdr[s_cur].period
    else
        let p_xperdr[p_cur].balanced = "N"
        display p_xperdr[p_cur].balanced to s_perdr[s_cur].balanced
        let n = p_xperdr[p_cur].period
        if n < 1 then let n = 1 end if     # no 0 periods entered
        if n is not null
        then
            let p_xperdr[p_cur].period = n using "&&"
            display p_xperdr[p_cur].period to s_perdr[s_cur].period
        end if
        let n = p_xperdr[p_cur].period_year

        # check for a 2 digit year, and convert to 4 digit
        if n is not null and n < 1000
        then
            if n > 50
              then 
                  let p_xperdr[p_cur].period_year = 
                      (n + 1900) using "&&&&"
              else 
                  let p_xperdr[p_cur].period_year = 
                      (n + 2000) using "&&&&"
            end if
            display p_xperdr[p_cur].period_year 
              to s_perdr[s_cur].period_year
        end if

        # check for duplicate entry
        if   p_xperdr[p_cur].period is not null
         and p_xperdr[p_cur].period_year is not null
        then
            for n = 1 to arr_count()
                if   n != p_cur       # don't dup check on this row
                 and p_xperdr[n].period = p_xperdr[p_cur].period
                 and p_xperdr[n].period_year = 
                     p_xperdr[p_cur].period_year
                then
                    call fg_err(6) # Duplicate period entered.
                    let p_xperdr[p_cur].period = null
                    let p_xperdr[p_cur].period_year = null
                    display
                        p_xperdr[p_cur].period, 
                        p_xperdr[p_cur].period_year
                      to
                        s_perdr[s_cur].period, 
                        s_perdr[s_cur].period_year
                    let nxt_fld = "period"
                    return
                end if  # dup found
            end for     # all rows
        end if          # period not null
    end if              # not g/l controlled

end function
# datechk()

#######################################################################
function close_setup()
# returning true/false based on setup completing or failing
#######################################################################
#  The sequence of events that occur during g/l init are:
#  - inform user & allow exit
#  - make sure current g/l period is defined in stxperdr table
#  - lock the chart of accounts table from update (during this process)
#  - verify the presence of the retained earnings account in stxchrtr
#  - make a retained earnings chart history table entry if necessary
#  - verify that all credits match all debits in chrt history table
#  - mark all period definitions as not under g/l control
#  - mark this period definition as being under g/l control
#  - build a cursor of all account balances in this period
#  - post beginning balances to g/l (found in the cursor)
#  - table unlocking will be done via a commit or rollback statement
#  - re-read the detail lines & display
#
    define
        c               char(1),        # working char
        x               smallint,       # working number
        o_journal like stgtranr.orig_journal,
        doc_date date,
        docno like stgtranr.doc_no,
        m_xchrtd record like stxchrtd.*,# working record like stxchrtd
        row_id          integer,        # working rowid
        db_cr           char(2),        # CR or DB for credit/debit
        scratch         char(70),       # scratch string storage
        all_credits     decimal(14),    # total of all credits
        all_debits      decimal(14)     # total of all debits

    open window end_setup at 14,16 with 9 rows, 51 columns
        attribute (border, blue, prompt line last)
    display "===================================================" at 2,1
    display " You have indicated that the general ledger setup  " at 3,1
    display " is complete.  If you wish to continue, the total  " at 4,1
    display " credits and debits will be summed and displayed.  " at 5,1
    display " You will then be given the opportunity to go on   " at 6,1
    display " or to quit.                                       " at 7,1
    display "===================================================" at 8,1
    let c = null
    while c matches "[^yYnN]" or
          c is null
        prompt " Continue? " for c
    end while
    if upshift(c) = "N"
    then
        close window end_setup
        return false
    end if

    call cls_window()
    display " Verifying data integrity...     please wait.      " at 1,1

    #  - lock the chart of accounts table from update
    display " - locking the account and transaction tables      " at 4,1
    sleep 1
    whenever error continue
    lock table stxchrtr in share mode
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        call fg_err(11) # Someone is modifying an account at this time.
        close window end_setup
        return false
    end if
    whenever error continue
    lock table stxchrtd in share mode
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        call fg_err(11) # Someone is modifying an account at this time.
        close window end_setup
        return false
    end if
    whenever error continue
    lock table stxtranr in share mode
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        call fg_err(11) # Someone is modifying an account at this time.
        close window end_setup
        return false
    end if
    whenever error continue
    lock table stgtranr in share mode
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        call fg_err(11) # Someone is modifying an account at this time.
        close window end_setup
        return false
    end if
    whenever error continue
    lock table stgactvd in share mode
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        call fg_err(11) # Someone is modifying an account at this time.
        close window end_setup
        return false
    end if

    #  - verify the presence of the retained earnings account
    display " - verifying the retained earnings account         " at 5,1
    sleep 1
    select count(*) into n from stxchrtr
      where acct_no = p_gcntrc.retain_earnings
    if n != 1 or sqlca.sqlcode != 0
    then
        call fg_err(12) # Cannot find retained earnings ledger account
        close window end_setup
        return false
    end if

    # make an entry into the retained earnings history table if not
    # already there for the 000 department.
    select count(*) into n from stxchrtd
        where acct_no = p_gcntrc.retain_earnings
          and department = "000"
          and period_month = p_gcntrc.curr_period
          and period_year = p_gcntrc.curr_year
    if n = 0 or sqlca.sqlcode != 0
    then
        whenever error continue
        insert into stxchrtd 
          (acct_no, department, period_month, period_year, 
           activity, balance, this_month, budget) 
        values
          (m_gcntrc.retain_earnings, "000", p_gcntrc.curr_period, 
           p_gcntrc.curr_year, "",0,"","")
        whenever error call error_handler
        if sqlca.sqlcode != 0
        then
            call fg_err(13) # Cannot insert retained earnings data.
            close window end_setup
            return false
        end if
    end if

    #  - verify that all credits match all debits in chrt history table
    display " - making sure the ledger accounts balance         " at 6,1
    sleep 1
    let all_credits = 0
    let all_debits  = 0

    declare c_balance cursor for
        select rowid, * from stxchrtd
          where period_month = p_gcntrc.curr_period
            and period_year = p_gcntrc.curr_year
    open c_balance
    foreach c_balance into row_id, m_xchrtd.*
        # determine CR/DB
        call al_chrtn(m_xchrtd.acct_no)
            returning scratch, db_cr
        if scratch = "NOT FOUND"
        then
            # found an orphan.  delete it.
            # (don't tell the child protective services)
            whenever error continue
            delete from stxchrtd where rowid = row_id
            whenever error call error_handler
            continue foreach
        end if
        if db_cr = "CR"
          then let all_credits = all_credits + m_xchrtd.balance
          else let all_debits  = all_debits + m_xchrtd.balance
        end if
    end foreach
    close c_balance
    call cls_window()

    if all_credits = all_debits
    then else
        display " The ledger accounts are not in balance." at 1,1
        let scratch = all_debits using  "Total Debits:  -<<<,<<<,<<<.&&"
        display scratch at 3,2
        let scratch = all_credits using "Total Credits: -<<<,<<<,<<<.&&"
        display scratch at 4,2
        display " The ledger accounts must be in balance." at 5,1
        display " You may run a trial balance report if you wish" at 6,1
        display " to view the balance of all ledger accounts." at 7,1
        prompt  " Press [ENTER] to continue: " for char c
        close window end_setup
        return false
    end if

    #  - mark all period definitions as not under g/l control
    #  - mark this period definition as being under g/l control
    display " - marking period definitions under G/L control" at 4,1
    sleep 1
    # make sure period definitions are valid
    if not valid_periods()
    then
        close window end_setup
        return false
    end if
    # remove and reinsert the current period definitions
    call lld_delete()
    call lld_add()
    whenever error continue
    update stxperdr set balanced = "N"
    update stxperdr set balanced = "Y"
      where period = p_gcntrc.curr_period
        and period_year = p_gcntrc.curr_year
    whenever error call error_handler

    #  - build a cursor of all account balances in this period
    display " - selecting ledger balances for posting" at 5,1
    sleep 1
    whenever error continue
    declare r_curs cursor for
        select * from stxchrtd
          where period_month = p_gcntrc.curr_period
            and period_year = p_gcntrc.curr_year
    open r_curs
    whenever error call error_handler

    #  - post beginning balances to g/l (found in the cursor)
    display " - posting balances to general ledger   " at 6,1
    sleep 1

    # set the document date to the start_date of this period
    select start_date into doc_date from stxperdr
      where period = p_gcntrc.curr_period
        and period_year = p_gcntrc.curr_year

    # bump up the general journal doc_no and post_no
    let m_gcntrc.genjrn_doc_no = m_gcntrc.genjrn_doc_no + 1
    let m_gcntrc.genjrn_post_no = m_gcntrc.genjrn_post_no + 1

    # main posting loop
    foreach r_curs into m_xchrtd.*
        call al_chrtn(m_xchrtd.acct_no)
            returning scratch, db_cr
        if gl_post("POST","GJ",m_gcntrc.genjrn_doc_no,
             m_gcntrc.genjrn_post_no, today, doc_date, "BEGBAL",
             "BEGINNING BALANCE ENTRY","BEGBAL",m_xchrtd.acct_no,
             m_xchrtd.department, m_xchrtd.balance, db_cr) = false
        then
            call fg_error("lib_all","gl_post",post_gl.status)
            close window end_setup
            return false
        end if
    end foreach
    if gl_last() = false
    then
        call fg_error("lib_all","gl_last",post_gl.status)
        close window end_setup
        return false
    end if

    #  - mark activity as posted.
    display " - marking activity as posted           " at 7,1
    sleep 1
    whenever error continue
    update stgtranr set status = "P" 
      where orig_journal = "GJ" and doc_no = m_gcntrc.genjrn_doc_no
    whenever error call error_handler
    if sqlca.sqlcode != 0
    then
        call fg_err(15) # Cannot mark activity as posted.
        close window end_setup
        return false
    end if

    #  - re-read the detail lines & display
    call lld_read()
    close window end_setup
    call lld_display()
    return true
end function
# close_setup()

#####################################################################
function cls_window()
#####################################################################
#  This function clears the middle portion of the window.
#
    display "" at 1,1
    display "" at 3,1
    display "" at 4,1
    display "" at 5,1
    display "" at 6,1
    display "" at 7,1
    return
end function
# cls_window()

#######################################################################
function valid_periods()
# returning true/false based on setup of periods is valid
#######################################################################
#  All periods that are defined after the current period will eventually
#  be set to under g/l control.
#  This function will check to assure that the all those period's 
#  date ranges are contagious and non overlapping.
#
    define
      prev record
          period char(2), 
          period_year char(4),
          start_date date, 
          end_date date,
          balanced char(1)
      end record,
      create_stmt      char(256),
      prev_period      char(2),
      prev_year        char(4),
      prev_end_date    date,
      curr_end_date    date,      # current period's ending date
      first_row        smallint

    open window valid_period at 12,14 with 1 rows, 55 columns
        attribute (border, blue)

    display " Verifying period definition integrity... please wait." at 1,1
    call mld_arr_count()  # get the number of periods that are defined

    let curr_end_date = null

    # get the end date of the current period
    for n = 1 to rec1_cnt
      if p_xperdr[n].period = p_gcntrc.curr_period and 
         p_xperdr[n].period_year = p_gcntrc.curr_year
      then
          let curr_end_date = p_xperdr[n].end_date
          exit for
      end if
    end for

    if curr_end_date is null
    then
        # Current period not defined.
        call fg_err(10)
        close window valid_period
        return false
    end if

    if tmp_tbl_created is null
    then
        let tmp_tbl_created = "Y"
        let create_stmt = 
          "create temp table per_temp ",
            "(period char(2), period_year char(4), ",
            "start_date date, end_date date, balanced char(1))"
        if is_4_0()
        then
            let create_stmt = create_stmt clipped, " with no log"
        end if
        prepare create_prep from create_stmt
        execute create_prep
    end if

    delete from per_temp where 1=1
    declare tmp_period cursor for
      select * from per_temp where end_date >= curr_end_date
        order by start_date
 
    for n = 1 to rec1_cnt
      insert into per_temp values(p_xperdr[n].*)
    end for

    let prev.period = p_gcntrc.curr_period
    let prev.period_year = p_gcntrc.curr_year
    let prev.end_date = curr_end_date

    let first_row = true
    foreach tmp_period into m_xperdr.*
      if first_row
      then
          let prev.* = m_xperdr.*
          let first_row = false
          continue foreach
      end if
      if (prev.end_date + 1) != m_xperdr.start_date
      then
          # start date out of sequence or overrlapping
          close window valid_period
          call fg_err(16)
          return false
      end if
      if prev.period_year = m_xperdr.period_year
      then
          if (prev.period + 1) != (m_xperdr.period + 0)
          then
              # period is out of sequence
              close window valid_period
              call fg_err(17)
              return false
          end if
      else
          if (prev.period_year + 1) != (m_xperdr.period_year + 0)
          then
              # period year is out of sequence
              close window valid_period
              call fg_err(18)
              return false
          else
              if m_xperdr.period != "01"
              then
                  # starting period of new year must be "01"
                  close window valid_period
                  call fg_err(19)
                  return false
              end if
          end if
      end if
      let prev.* = m_xperdr.*
    end foreach

    close window valid_period
    return true

end function 
# valid_periods()

#######################################################################
function getrec1()
#######################################################################
#  this function is for control files.  it creates a new record if
#  no records are found.  this needs to be done because ok_add should
#  be set to return false.
#
    define
        num_rows    smallint
 
    select count(*) into num_rows from stgcntrc
    if num_rows < 1
    then
        initialize m_gcntrc.* to null
        let m_gcntrc.gl_balanced = "N"
        insert into stgcntrc values(m_gcntrc.*)
    end if

end function
# getrec1()

######################################################################
function lld_m_prep(n)
######################################################################
# This function creates the m_* record from the specified element
# of the p_.* array, and from elements of the header that tie the
# detail lines to the header
#
    define
        n smallint      # Array element to process

    # Items from the p_ record array...
    let m_xperdr.period = p_xperdr[n].period
    let m_xperdr.period_year = p_xperdr[n].period_year
    let m_xperdr.start_date = p_xperdr[n].start_date
    let m_xperdr.end_date = p_xperdr[n].end_date
    let m_xperdr.balanced = p_xperdr[n].balanced
end function
# lld_m_prep()

######################################################################
function lld_read()
######################################################################
# This function reads in the array elements from disk.
# It first reads the data into the m_* records, then calls
# p_prep for each array element.
#
    define
        row_id integer  # Rowid of the fetched element

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select rowid, * from stxperdr ",
          "where stxperdr.period != \"00\"",
          " order by period_year desc, period desc"
        prepare read_specs from scratch
        declare d_cursor cursor for read_specs
    end if

    # Read in the detail lines
    let p_cur = 0
    open d_cursor
    foreach d_cursor into row_id, m_xperdr.*
        let p_cur = p_cur + 1
        if p_cur > rec1_max
        then
            let sqlca.sqlcode = -1
            return
        end if
        call lld_p_prep(p_cur)
        let q_xperdr[p_cur].row_id = row_id
    end foreach
    close d_cursor
    let rec1_cnt = p_cur
    call set_count(rec1_cnt)
end function
# lld_read()

######################################################################
function lld_delete()
######################################################################
# This function deletes the rows in the detail table
# that match the where clause
#
    delete from stxperdr
      where stxperdr.period != "00"
end function
# lld_delete()

