######################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  %Z%  %M%  %I%  Delta: %G%
######################################################################

globals "globals.4gl"

# Local (static) variables
define
    lookup_prep char(1), # Have the lookups been prepared?
    select_prep char(1), # Has the select statement been prepared?
    exit_level smallint, # 0=input, 1=field
    tab_pressed smallint # (boolean) was the tab key pressed?


######################################################################
function llh_input()
# returning -1 if tab pressed (next window), 0 otherwise
######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    # Initialize some variables
    let exit_level = 0
    let tab_pressed = false


    input p_gcntrc.* without defaults from s_cntrc.*

      # All entry fields must have before and after field processing
      before field curr_period call llh_b_field("stgcntrc.curr_period")
        goto next_field
      before field curr_year call llh_b_field("stgcntrc.curr_year")
        goto next_field
      before field retain_earnings call llh_b_field("stgcntrc.retain_earnings")
        goto next_field
      before field dir_db_cr call llh_b_field("stgcntrc.dir_db_cr")
        goto next_field
      before field setup_date call llh_b_field("stgcntrc.setup_date")
        goto next_field
      before field gl_balanced call llh_b_field("stgcntrc.gl_balanced")
        goto next_field

      after field curr_period call llh_a_field()
        goto next_field
      after field curr_year call llh_a_field()
        goto next_field
      after field retain_earnings call llh_a_field()
        goto next_field
      after field dir_db_cr call llh_a_field()
        goto next_field
      after field setup_date call llh_a_field()
        goto next_field
      after field gl_balanced call llh_a_field()
        goto next_field

      after input
        label end_input:
        let nxt_fld = null

        # Exit directly upon interrupt
        if int_flag then exit input end if

        # Run the 'after field' logic if necessary
        if exit_level >= 1 then call llh_a_field() end if
        if nxt_fld is not null then goto next_field end if

        # Run the 'after input' logic before exiting input
        call llh_a_input()
        if nxt_fld is not null then goto next_field end if
        exit input

      # Event trapping logic
      on key (control-b) let hotkey = 2  goto event
      on key (control-e) let hotkey = 5  goto event
      on key (control-f) let hotkey = 6  goto event
      on key (control-g) let hotkey = 7  goto event
      on key (control-i) let hotkey = 9  goto event
      on key (control-n) let hotkey = 14 goto event
      on key (control-o) let hotkey = 15 goto event
      on key (control-p) let hotkey = 16 goto event
      on key (control-t) let hotkey = 20 goto event
      on key (control-u) let hotkey = 21 goto event
      on key (control-v) let hotkey = 22 goto event
      on key (control-w) let hotkey = 23 goto event
      on key (control-y) let hotkey = 25 goto event
      on key (control-z) let hotkey = 26 goto event
      on key (f5)  let hotkey = 105 goto event
      on key (f6)  let hotkey = 106 goto event
      on key (f7)  let hotkey = 107 goto event
      on key (f8)  let hotkey = 108 goto event
      on key (f9)  let hotkey = 109 goto event
      on key (f10) let hotkey = 110 goto event
      on key (f11) let hotkey = 111 goto event
      on key (f12) let hotkey = 112 goto event
      on key (f13) let hotkey = 113 goto event
      on key (f14) let hotkey = 114 goto event
      on key (f15) let hotkey = 115 goto event
      on key (f16) let hotkey = 116 goto event
      on key (f34) let hotkey = 134 goto event
      on key (f35) let hotkey = 135 goto event
      on key (f36,interrupt) let hotkey = 136 goto event

      # Local event processing
      label event:
        call hot_key("llh_input")  # Map the key to the event
        call llh_event()

      # Programmatic next field logic
      label next_field:
        case
          when nxt_fld is null  # No need to go through 'case'
          when nxt_fld = "curr_period" next field curr_period
          when nxt_fld = "curr_year" next field curr_year
          when nxt_fld = "retain_earnings" next field retain_earnings
          when nxt_fld = "dir_db_cr" next field dir_db_cr
          when nxt_fld = "setup_date" next field setup_date
          when nxt_fld = "gl_balanced" next field gl_balanced
          when nxt_fld = "exit input" goto end_input
        end case
    end input

    # Return -1 if tab was pressed
    if tab_pressed
      then return -1
      else return 0
    end if
end function
# llh_input()

######################################################################
function llh_b_field(field_name)
######################################################################
# This function is called from the input function before every field.
# The 'prv_fld' variable contains the field we came from.
# The 'scr_fld' variable contains the field we're going into.
# Set 'nxt_fld' if you want to skip this field or exit input.
#
    define
        field_name char(40),  # Fieldname passed
        prv_fld char(40)      # The field we were just in

    # Convert field_name from 'tab.col' to 'col'
    let field_name = set_fldtab(field_name)

    # Rearrange field names
    let prv_fld = scr_fld
    let scr_fld = field_name
    let nxt_fld = null

    # Require field level exit
    let exit_level = 1

    # Specify fields only if work needs to be done.
    case
      when scr_fld = "setup_date"
        if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "Y"
          then let nxt_fld = "dir_db_cr" end if
      when scr_fld = "curr_period"
        if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "Y"
          then let nxt_fld = "dir_db_cr" end if
      when scr_fld = "retain_earnings"
        if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "Y"
          then let nxt_fld = "dir_db_cr" end if
    let scratch = "zoom"
      when scr_fld = "dir_db_cr"
        if p_gcntrc.dir_db_cr is null
          then let p_gcntrc.dir_db_cr = "N" end if
      when scr_fld = "gl_balanced"
        if p_gcntrc.gl_balanced is null
          then let p_gcntrc.gl_balanced = "N" end if
        if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "Y"
          then let nxt_fld = "dir_db_cr" return end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # Setup for lib_before
    let scr_fld = prv_fld
    call lib_before(field_name)

end function
# llh_b_field()

######################################################################
function llh_a_field()
######################################################################
# This function is called after every field.
#
    # Reset nxt_fld
    let nxt_fld = null

    # No more exit levels required
    let exit_level = 0

    # After field logic
    case
      when scr_fld = "curr_period"
        call fgha_curr_period()
        if p_gcntrc.curr_period = "00"
          then let nxt_fld = "curr_period" end if
      when scr_fld = "curr_year"
        call fghb_curr_year()
      when scr_fld = "retain_earnings"
        let scratch =
          " stxchrtr.acct_cat = \"C\""
        if auto_zoom("chrtz","stxchrtr","(see scratch)")
          then let p_gcntrc.retain_earnings = scratch end if
      when scr_fld = "dir_db_cr"
        if (p_gcntrc.dir_db_cr != "Y" and
            p_gcntrc.dir_db_cr != "N") or
            p_gcntrc.dir_db_cr is null
        then
            let p_gcntrc.dir_db_cr = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "dir_db_cr"
            return
        end if
      when scr_fld = "gl_balanced"
        if (p_gcntrc.gl_balanced != "Y" and
            p_gcntrc.gl_balanced != "N") or
            p_gcntrc.gl_balanced is null
        then
            let p_gcntrc.gl_balanced = null
            call lib_error("lib_scr", "lookup", 1, "")
            let nxt_fld = "gl_balanced"
            return
        end if
    end case

    # Programmed exit
    if nxt_fld is not null then return end if

    # This (among other things) sets data_changed
    call lib_after()

    # After data_changed logic
    if data_changed
    then
        case
          when scr_fld = "retain_earnings"
            # Perform lookups
            if llh_lookup("ret_acct",true) = false and
              length(this_data) != 0
            then
                let nxt_fld = "retain_earnings"
                return
            end if
            call fgha_retain_earnings()
        end case
    end if

end function
# llh_a_field()

######################################################################
function llh_a_input()
######################################################################
# This function is called whenever the input statement exits
# (except due to an interrupt).  If you don't want the input session
# to end, set the nxt_fld variable to contain the field to be placed
# back into.
#
    if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "N"
    then
        if mustfill(p_gcntrc.curr_period,"Current Period")
          then let nxt_fld = "curr_period" return end if
        if mustfill(p_gcntrc.curr_year,"Current Year")
          then let nxt_fld = "curr_year" return end if
        if mustfill(p_gcntrc.retain_earnings,"Retained Earnings")
          then let nxt_fld = "retain_earnings" return end if
        if mustfill(p_gcntrc.dir_db_cr,"Direct DB/CR Entry")
          then let nxt_fld = "dir_db_cr" return end if
        if mustfill(p_gcntrc.setup_date,"Setup Complete Date")
          then let nxt_fld = "setup_date" return end if
        if  p_gcntrc.retain_earnings >= m_xcntrc.income
         or p_gcntrc.retain_earnings <  m_xcntrc.capital
        then
            # Retain earning must be in Capital range.
            call fg_err(1)
            let nxt_fld = "curr_period"
            return
        end if
    end if
    if not tab_pressed
    then
        if not valid_periods()
          then let tab_pressed = true return end if
        if p_gcntrc.gl_balanced = "Y" and m_gcntrc.gl_balanced = "N"
        then
            if close_setup() = false
            then
                let p_gcntrc.gl_balanced = "N"
                call ok_cancel("Y")
                let int_flag = -1
            end if
        end if
    end if

end function
# llh_a_input()

######################################################################
function llh_event()
######################################################################
# This function is called whenever the user presses an event key.
# The event is mapped to the 'scr_funct' variable and processed here.
#
    # Reset tab pressed to false
    let tab_pressed = false
    case
      when scr_funct = "zoom" and infield(retain_earnings)
        let scratch =
          " stxchrtr.acct_cat = \"C\""
        if zoom("chrtz","(see scratch)")
        then
            let p_gcntrc.retain_earnings = scratch
            let nxt_fld = "retain_earnings"
        end if
      when scr_funct = "accept"
        let nxt_fld = "exit input"
      when scr_funct = "tab" or scr_funct = "btab"
        let tab_pressed = true
        let nxt_fld = "exit input"
      when scr_funct = "cancel"
        if no_cancel()
        then
          let int_flag = false
          let nxt_fld = null
        else
          let int_flag = true
          let nxt_fld = "exit input"
        end if
    end case
    let scr_funct = ""

end function
# llh_event()

######################################################################
function llh_setdata()
######################################################################
# This function is called to set the this_data global variable.
#
    case
      when scr_fld = "curr_period"
        call set_data(p_gcntrc.curr_period)
      when scr_fld = "curr_year"
        call set_data(p_gcntrc.curr_year)
      when scr_fld = "retain_earnings"
        call set_data(p_gcntrc.retain_earnings)
      when scr_fld = "dir_db_cr"
        call set_data(p_gcntrc.dir_db_cr)
      when scr_fld = "setup_date"
        call set_data(p_gcntrc.setup_date)
      when scr_fld = "gl_balanced"
        call set_data(p_gcntrc.gl_balanced)
    end case
end function
# llh_setdata()

######################################################################
function llh_high()
######################################################################
# This function highlights the specified field name.
# Only input type fields need to be specified.
#
    case
      when scr_fld = "curr_period"
        display by name p_gcntrc.curr_period attribute(reverse)
      when scr_fld = "curr_year"
        display by name p_gcntrc.curr_year attribute(reverse)
      when scr_fld = "retain_earnings"
        display by name p_gcntrc.retain_earnings attribute(reverse)
      when scr_fld = "dir_db_cr"
        display by name p_gcntrc.dir_db_cr attribute(reverse)
      when scr_fld = "setup_date"
        display by name p_gcntrc.setup_date attribute(reverse)
      when scr_fld = "gl_balanced"
        display by name p_gcntrc.gl_balanced attribute(reverse)
    end case
end function
# llh_high()

######################################################################
function llh_display()
######################################################################
# This function displays all p_* variables onto the screen
#
    display p_gcntrc.* to s_cntrc.*  attribute(red)
end function
# llh_display()

######################################################################
function llh_p_prep()
######################################################################
# This function creates the p_* record from the m_* record
#
    let p_gcntrc.curr_period = m_gcntrc.curr_period
    let p_gcntrc.curr_year = m_gcntrc.curr_year
    let p_gcntrc.retain_earnings = m_gcntrc.retain_earnings
    let p_gcntrc.dir_db_cr = m_gcntrc.dir_db_cr
    let p_gcntrc.setup_date = m_gcntrc.setup_date
    let p_gcntrc.gl_balanced = m_gcntrc.gl_balanced

    # Perform lookups
    if llh_lookup("ret_acct",false) then end if

end function
# llh_p_prep()

######################################################################
function llh_m_prep()
######################################################################
# This function creates the m_* record from the p_* record
#
    let m_gcntrc.curr_period = p_gcntrc.curr_period
    let m_gcntrc.curr_year = p_gcntrc.curr_year
    let m_gcntrc.retain_earnings = p_gcntrc.retain_earnings
    let m_gcntrc.dir_db_cr = p_gcntrc.dir_db_cr
    let m_gcntrc.setup_date = p_gcntrc.setup_date
    let m_gcntrc.gl_balanced = p_gcntrc.gl_balanced
end function
# llh_m_prep()

######################################################################
function llh_read(ring_rowid)
######################################################################
# This function loads the header p_* record elements.
# It first loads the m_* record from disk, then it
# calls the p_prep routine to fill the p_* record elements.
#
    define
        ring_rowid  integer     # rowid of record to read

    # Prepare the cursor upon first invocation
    if select_prep is null
    then
        let select_prep = "Y"
        let scratch = "select * from stgcntrc where rowid = ?"
        prepare read_specs from scratch
        declare h_cursor cursor for read_specs
    end if

    # Read the header record
    open h_cursor using ring_rowid
    fetch h_cursor into m_gcntrc.*
    if sqlca.sqlcode != 0 then return end if
    let q_gcntrc.row_id = ring_rowid
    close h_cursor
    call llh_p_prep()
end function
# llh_read()

######################################################################
function llh_delete(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    delete from stgcntrc
        where rowid = ring_rowid
end function
# llh_delete()

######################################################################
function llh_add()
######################################################################
# This function inserts data into the header table.
#
    define
        new_rowid integer  # Rowid after insert
    # Insert the data
    insert into stgcntrc values(m_gcntrc.*)
    let new_rowid = sqlca.sqlerrd[6]

    # Reset rowid
    let sqlca.sqlerrd[6] = new_rowid

end function
# llh_add()

######################################################################
function llh_update(ring_rowid)
######################################################################
#
    define
        ring_rowid  integer     # rowid passed
    update stgcntrc set stgcntrc.* = m_gcntrc.*
      where rowid = ring_rowid
end function
# llh_update()

######################################################################
function llh_lookup(tbl_name,mustfind)
# returning false if notfound
######################################################################
# This is the lookup function.  It is passed a table to lookup, and
# based on that, it looks up the key and fills any variables for
# that table.  The lookup function will produce the error if it is
# called with mustfind=true, and a notfound situation exists.
#
    define
        tbl_name char(18),  # Lookup table name
        stat integer,       # Value of sqlca.sqlcode after lookup
        mustfind smallint   # (boolean) produce an error if notfound?

    # Prepare cursors upon first call to this function
    if lookup_prep is null
    then
        let lookup_prep = "Y"
        let scratch = "select rowid ",
          "from stxchrtr where ",
          " stxchrtr.acct_cat = \"C\" and",
          " stxchrtr.acct_no = ?"
        prepare str_ret_acct from scratch
        declare cur_ret_acct cursor for str_ret_acct
    end if

    # Perform the lookup
    case
      when tbl_name = "ret_acct"
        open cur_ret_acct using p_gcntrc.retain_earnings
        fetch cur_ret_acct into scratch
        let stat = sqlca.sqlcode
        close cur_ret_acct
    end case

    # No mustfind if this_data is empty
    if length(this_data) = 0 then let mustfind = false end if

    # Send notfound message to user (if requested)
    let sqlca.sqlcode = stat
    if mustfind and stat = notfound
      then call scr_error("lookup=notfound",tbl_name)
    end if
    let sqlca.sqlcode = 0

    # Return false if lookup failed and empty into fields
    if stat = notfound
    then
        case
          when tbl_name = "ret_acct"
            initialize scratch to null
        end case
        return false
    else
        return true
    end if
end function
# llh_lookup()

