########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../gl.4gm/o_balsht.4gs/lowlevel.4gl  1.4  Delta: 1/7/91
########################################################################

globals "globals.4gl"

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    # Trap fatal errors
    whenever error call error_handler

    let rpt.subtotal_group = curs.subtotal_group
    let rpt.incr_with_crdt = curs.incr_with_crdt
    let rpt.period_year = curs.period_year
    let rpt.period_month = curs.period_month
    let rpt.balance = curs.balance
    let rpt.this_month = curs.this_month
#   let rpt.acct_period = (MUST SUPPLY)
#   let rpt.asterisk = (MUST SUPPLY)
#   let rpt.note = (MUST SUPPLY)
#   let rpt.dept = (MUST SUPPLY)
#   let rpt.category = (MUST SUPPLY)
    let rpt.acct_type = curs.acct_type
#   let rpt.sub_group = (MUST SUPPLY)
    let rpt.acct_no = curs.acct_no
    let rpt.department = curs.department
    let rpt.acct_desc = curs.acct_desc
#   let rpt.amount = (MUST SUPPLY)
#   let rpt.sub_amt = (MUST SUPPLY)
    let rpt.acct_cat = curs.acct_cat
    let rpt.processing_seq = curs.processing_seq

    # display accounting period at top of every page
    let rpt.acct_period = accum.acct_period

    # asterisk or no asterisk
    let rpt.asterisk = ""
    if accum.this_period = true
    then
        select count(*) into accum.this_period from stxchrtd where
            acct_no = curs.acct_no and
            department = curs.department and
            period_month != accum.acct_mo and
            period_year  != accum.acct_yr and
            this_month is not null
        if accum.this_period != 0
        then
            let rpt.asterisk = "*"
        end if
        let accum.this_period = true
    else
        if curs.this_month is not null
        then
            let rpt.asterisk = "*"
        end if
    end if

    # set department name
    let rpt.dept = accum.dept

    # set the headings:
    if curs.acct_cat = "A"
    then
        let rpt.category = "ASSETS"
    else
        let rpt.acct_cat = "B"
        let rpt.category = "LIABILITIES & CAPITAL"
    end if

    # set the amount...
    if  (curs.acct_cat  = "A" and curs.incr_with_crdt != "Y")
     or (curs.acct_cat != "A" and curs.incr_with_crdt  = "Y")
    then
        # normal account
        let rpt.amount = curs.balance
    else
        # contra account - show as a negative on the balance sheet
        let rpt.amount = curs.balance * "-1"
    end if

    # accumulate subtotals
    let accum.sub_amt = accum.sub_amt + rpt.amount

    return
end function
# on_every_row()

#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "first_row" then
        call gcontrol() returning p_gcntrc.*
        if accum.dept is null or accum.dept = "CONSOLIDATED"
        then
            # calculate the p&l amount for this period
            call getpl()
        end if
        return
    end if

    if group_key = "acct_cat" then
        # insert "before group of acct_cat" code here
        return
    end if

    if group_key = "processing_seq" then
        # insert "before group of processing_seq" code here
        return
    end if

    if group_key = "acct_no" then
        let rpt.sub_group = ""
        if curs.subtotal_group = accum.sub_group
           or curs.subtotal_group is null
           or curs.subtotal_group = " "
        then else
            # first row of non_null subtotal group
            let rpt.sub_group = curs.subtotal_group
            let accum.sub_amt = 0
        end if
        let accum.sub_group = curs.subtotal_group
        if   (curs.acct_no = p_gcntrc.retain_earnings)
         and (accum.dept is null or accum.dept = "CONSOLIDATED")
        then
            # print out the p&l line
            let curs.balance = accum.pl_amount
            let curs.acct_no = ""
            let curs.department = ""
            let curs.acct_desc = "Y.T.D. EARNINGS"
            let rpt.sub_amt = ""
            # process this line like all the rest
            call on_every_row()
            # send it to the report
            call ml_output()
            # bring the curs record back to normal
            call ml_curs_prep()
            # the effect of all of this is to print out
            # a p/l line right before the retained earnings
            # line (the p/l line isn't a row in the chart of accounts,
            # it must be calculated on the fly for each report.)
        end if
        return
    end if

    if group_key = "department" then
        # insert "before group of department" code here
        return
    end if

end function
# before_group()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "acct_cat" then
        # insert "after group of acct_cat" code here
        return
    end if

    if group_key = "processing_seq" then
        # insert "after group of processing_seq" code here
        return
    end if

    if group_key = "acct_no" then
        if curs_next.subtotal_group = curs.subtotal_group
           or curs.subtotal_group is null
           or curs.subtotal_group = " "
        then
            let rpt.sub_amt = ""
        else
            # last row of non-null subtotal group
            let rpt.sub_amt = accum.sub_amt
        end if
        return
    end if

    if group_key = "department" then
        # insert "after group of department" code here
        return
    end if

    if group_key = "last_row" then
        let rpt.note = accum.note
        return
    end if

end function
# after_group()

#######################################################################
function getpl()
#######################################################################
# this function gets the ytd p/l amount for the reporting period.
#
    define
        cat like stxchrtr.acct_cat,
        crdt like stxchrtr.incr_with_crdt,
        bal like stxchrtd.balance

    let accum.pl_amount = 0
    let status = 0
    declare pl_curs cursor for
        select acct_cat, incr_with_crdt, balance
            from stxchrtr, stxchrtd
            where stxchrtr.acct_no = stxchrtd.acct_no
              and acct_cat <= "C"
              and stxchrtd.period_month = accum.acct_mo
              and stxchrtd.period_year  = accum.acct_yr
    open pl_curs
    foreach pl_curs into cat, crdt, bal
        if crdt = "N"
        then
            let accum.pl_amount = accum.pl_amount + bal
        else
            let accum.pl_amount = accum.pl_amount - bal
        end if
    end foreach
    close pl_curs
    return
end function
# getpl()
